require 'rails_helper'
require 'support/model_classes'
require 'support/factories'
require 'support/setup'
require 'pry'


describe "group model mixin" do

  context "Fail to find a group" do
    it "should raise an error on trying to find non existent group" do
      expect{
        Group.find(1)
      }.to raise_error ActiveRecord::RecordNotFound
    end
  end

  context "Validation checks" do
    it "validates a group object" do
      FactoryGirl.build(:member_group).should be_valid
    end

    it "should not validate a group object, based on an empty name" do
      FactoryGirl.build(:member_group, group_name: '').should_not be_valid
    end
  end

  context "Friendly ID" do
    it "should find based on name" do
      @group = FactoryGirl.create(:member_group)
      Group.find(@group.group_name.parameterize).should_not eql nil
    end
  end

  context "Create role relationship" do
    it "create a relationship and check that it exists" do
      @group = FactoryGirl.create(:member_group)
      @role = FactoryGirl.create(:admin_role)
      @group.add_role = @role.role_name
      @group.has_role?(@role.role_name).should eql true
    end

    it "should return an array of roles for a group" do
      @group = FactoryGirl.create(:member_group)
      @role = FactoryGirl.create(:admin_role)
      @group.add_role = @role.role_name
      @group.roles.empty?.should eql false
    end

    it "should return false for a group that cannot be altered" do
      @group = FactoryGirl.create(:member_group, can_alter: false)
      group = Group.find(@group.group_name.parameterize)
      group.can_alter?.should eql false
    end

    it "should return false for a group that cannot be deleted" do
      @group = FactoryGirl.create(:member_group, can_delete: false)
      group = Group.find(@group.group_name.parameterize)
      group.can_delete?.should eql false
    end
  end

  describe "Roles" do
    it "should get all roles for a group" do
      @group = FactoryGirl.create(:member_group)
      @role = FactoryGirl.create(:admin_role)
      @group.add_role = @role.role_name
      @group.role_names.empty?.should_not eql true
    end
  end

  describe "Concerns" do
    before(:each) do
      no_user_member_roles_relation
    end

    it "should have a role" do
      @group.has_role?(@role.role_name).should eql true
    end

    it "should get a role" do
      @group.get_role(@role.role_name).should_not eql nil
    end

    it "should get a set of roles" do
      @group.role_names.empty?.should_not eql true
    end

    it "should get a remove a role" do
      @group.remove_role = @role.role_name
      @group.has_role?(@role.role_name).should eql false
    end
  end
end
