require 'spec_helper'
require 'pry'

describe "role model mixin" do

  context "Fail to find a role" do
    it "should raise an error on trying to find non existent role" do
      expect{
        Role.find(1)
      }.to raise_error ActiveRecord::RecordNotFound
    end
  end

  context "Validation checks" do
    it "validates a role object" do
      FactoryGirl.build(:admin_role).should be_valid
    end

    it "should not validate a role object, based on an empty name" do
      FactoryGirl.build(:admin_role, role_name: '').should_not be_valid
    end
  end

  context "Friendly ID" do
    it "should find based on name" do
      @role = FactoryGirl.create(:admin_role)
      Role.find(@role.role_name.parameterize).should_not eql nil
    end
  end

  context "Belongs to a Group?" do
    before(:each) do
      @role = FactoryGirl.create(:admin_role)
      @group = FactoryGirl.create(:member_group, group_name: 'Administrator')
    end

    it "Should belong to a group" do
      @group.add_role = @role.role_name
      @role.has_group?(@group.group_name).should eql true
    end

    it "should not belong to a group" do
      @role.has_group?(@group.group_name).should eql false
    end
  end

  context "Create permission relationship" do
    it "create a relationship and check that it exists" do
      @role = FactoryGirl.create(:admin_role)
      @permission = FactoryGirl.create(:can_read)
      @role.add_permission = @permission.permission_name
      @role.has_permission?(@permission.permission_name).should eql true
    end

    it "should not return false for permissions added to a role" do
      @role = FactoryGirl.create(:admin_role)
      @permission = FactoryGirl.create(:can_read)
      @role.add_permission = @permission.permission_name
      @role.permissions.empty?.should eql false
    end

    it "should return an array of permissions for a role" do
      @role = FactoryGirl.create(:admin_role)
      @permission = FactoryGirl.create(:can_read)
      @role.add_permission = @permission.permission_name
      @role.permissions.empty?.should eql false
    end

    it "should return false for a role that cannot be altered" do
      @role = FactoryGirl.create(:admin_role, can_alter: false)
      role = Role.find(@role.role_name.parameterize)
      role.can_alter?.should eql false
    end

    it "should return false for a role that cannot be deleted" do
      @role = FactoryGirl.create(:admin_role, can_delete: false)
      role = Role.find(@role.role_name.parameterize)
      role.can_delete?.should eql false
    end
  end

  describe "Permissions" do
    it "should get a set of permissions for a role" do
      @role = FactoryGirl.create(:admin_role)
      @permission = FactoryGirl.create(:can_read)
      @role.add_permission = @permission.permission_name
      @role.permission_names.empty?.should_not eql true
    end
  end

  describe "Concerns" do
    before(:each) do
      no_user_member_roles_relation
    end

    it "should belong to the member group" do
      @role.has_group?('Member').should eql true
    end

    it "should have permissions" do
      @role.permissions.nil?.should eql false
    end

    it "should get permission names" do
      @role.permission_names.empty?.should_not be true
    end

    it "should delete a permission" do
      @role.remove_permission = @permission.permission_name
      @role.has_permission?(@permission.permission_name).should eql false
    end
  end
end
