require 'rails_helper'
require 'support/model_classes'
require 'support/factories'
require 'support/setup'
require 'pry'

describe "Test the user model mixin" do

  context "Fail to find a user" do
    it "should return nil on a user not existing" do
      User.find_by_first_name('sally').should == nil
    end
  end

  context "Validation checks" do
    it "Validate a valid user object" do
     FactoryGirl.build(:user).should be_valid
    end

    it "should not be valid if wong user name length" do
      FactoryGirl.build(:user, :user_name => 'gg').should_not be_valid
    end

    it "should not be valid if missing password" do
      FactoryGirl.build(:user, :password => '').should_not be_valid
    end

    it "should not be valid if wong password length" do
      FactoryGirl.build(:user, :password => 'ggg').should_not be_valid
    end

    it "should not be valid if wong email" do
      FactoryGirl.build(:user, :email => 'ggg.com').should_not be_valid
    end

    it "should validate a user based on login credentials" do
      @user = FactoryGirl.create(:user)
      @login = FactoryGirl.build(:user)
      User.authenticate_user(@user.user_name, @login.password).should == @user
    end

    it "should fail on password being wrong" do
      @user = FactoryGirl.create(:user)
      @login = FactoryGirl.build(:user)
      User.authenticate_user(@user.user_name, "skfdjhgsdkjfhg").should == nil
    end

    it "should hash my password" do
      @user = FactoryGirl.create(:user)
      @login = FactoryGirl.build(:user)
      @user.password.should == BCrypt::Engine.hash_secret(@login.password, @user.salt)
    end
  end

  context "Assign users groups, roles and permissions" do
  before(:each) do
    setup_group_role_permissions_relations_for_memeber
  end

  it "should assign a group to a user" do
    @user.add_group = @group.group_name
    @user.has_group?(@group.group_name).should_not eql false
  end

  it "should have a role for a user who belongs to a group" do
    @user.add_group_membership(@group, @role)
    @user.groups.find(@group.group_name.parameterize).has_role?(@role.role_name).should_not eql false
  end

  it "should have a permission for a role when a user who belongs to a group that has that role" do
    @user.add_group = @group.group_name
    @user.groups.find(@group.group_name.parameterize).roles.find(@role.role_name.parameterize).has_permission?(@permission.permission_name)
  end
end

context "Get a Group and Roles and Permissions" do
  before(:each) do
    setup_group_role_permissions_relations_for_memeber
  end

  it "should return one group a user belongs to" do
    @user.groups.nil?.should eql false
  end

  it "should return roles for a user belongs to a group" do
    @user.roles.empty?.should eql false
  end

  it "should return permissions for a user who has a role" do
    @user.permissions.empty?.should eql false
  end
end

context "Concerns" do
  before(:each) do
    setup_group_role_permissions_relations_for_memeber
  end

  it "should get a set of associated groups" do
    @user.group_names.should_not be_empty
  end

  it "should get a set of associated roles" do
    @user.role_names.should_not be_empty
  end

  it "should get unique permissions" do
    @user.uniq_permissions.should_not be_empty
  end

  it "should get all permissions" do
    @user.permissions.should_not be_empty
  end

  it "should return true when the role belongs to the group and the user has both" do
    @user.has_role_for_group?('Member', 'Member').should eql true
  end

  it "should get a role from a group association" do
    group = Group.find('member')
    @user.get_role_from_group_membership(group.id)
  end

  it "should remove a group membership association" do
    @user.remove_group_membership('Member')
    @user.has_group?('Member').should eql false
  end

  it "should get the first role" do
    @user.role.should eql  'Member'
  end

  it "should get the first group" do
    @user.group.should eql  'Member'
  end

end

end
